@@ -6,7 +6,7 @@
 # Written by Ilya Zverev, licensed WTFPL
 
 import mapnik
-import sys, os, argparse, math
+import sys, os, re, argparse, math
 
 try:
 	import cairo
@@ -46,13 +46,13 @@ def select_layers(m, enable, disable):
 		if l.name in disable:
 			l.active = False
 
-def prepare_ozi(m, name):
+def prepare_ozi(mbbox, mwidth, mheight, name):
 	"""Create georeferencing file for OziExplorer"""
 	def deg(value, is_lon):
 		degrees = math.floor(abs(value))
 		minutes = (abs(value) - degrees) * 60
 		return '{:4d},{:3.5F},{}'.format(int(round(degrees)), minutes, ('W' if is_lon else 'S') if value < 0 else ('E' if is_lon else 'N'))
-	bbox = transform.backward(m.envelope())
+	bbox = transform.backward(mbbox)
 	points = "\n".join(['Point{:02d},xy,     ,     ,in, deg,    ,        ,N,    ,        ,E, grid,   ,           ,           ,N'.format(n) for n in range(3,31)])
 	return '''OziExplorer Map Data File Version 2.2
 Nik4
@@ -73,27 +73,54 @@ Moving Map Parameters = MM?    These follow if they exist
 MM0,Yes
 MMPNUM,4
 MMPXY,1,0,0
-'''.format(name, deg(bbox.maxy, False), deg(bbox.minx, True), m.width - 1, m.height - 1, deg(bbox.miny, False), deg(bbox.maxx, True), points) \
-	+ "MMPXY,2,{},0\n".format(m.width) \
-	+ "MMPXY,3,{},{}\n".format(m.width, m.height) \
-	+ "MMPXY,4,0,{}\n".format(m.height) \
+'''.format(name, deg(bbox.maxy, False), deg(bbox.minx, True), mwidth - 1, mheight - 1, deg(bbox.miny, False), deg(bbox.maxx, True), points) \
+	+ "MMPXY,2,{},0\n".format(mwidth) \
+	+ "MMPXY,3,{},{}\n".format(mwidth, mheight) \
+	+ "MMPXY,4,0,{}\n".format(mheight) \
 	+ 'MMPLL,1,{:4.6f},{:4.6f}\n'.format(bbox.minx, bbox.maxy) \
 	+ 'MMPLL,2,{:4.6f},{:4.6f}\n'.format(bbox.maxx, bbox.maxy) \
 	+ 'MMPLL,3,{:4.6f},{:4.6f}\n'.format(bbox.maxx, bbox.miny) \
 	+ 'MMPLL,4,{:4.6f},{:4.6f}\n'.format(bbox.minx, bbox.miny) \
-	+ "MM1B,{}\n".format(m.scale() * math.cos(math.radians(bbox.center().y))) \
+	+ "MM1B,{}\n".format((mbbox.maxx - mbbox.minx) / mwidth * math.cos(math.radians(bbox.center().y))) \
 	+ "MOP,Map Open Position,0,0\n" \
-	+ "IWH,Map Image Width/Height,{},{}\n".format(m.width, m.height)
+	+ "IWH,Map Image Width/Height,{},{}\n".format(mwidth, mheight)
 
-def prepare_wld(m):
+def prepare_wld(bbox, mwidth, mheight):
 	"""Create georeferencing world file"""
-	bbox = transform.backward(m.envelope())
-	pixel_x_size = (bbox.maxx - bbox.minx) / m.width
-	pixel_y_size = (bbox.maxy - bbox.miny) / m.height
+	pixel_x_size = (bbox.maxx - bbox.minx) / mwidth
+	pixel_y_size = (bbox.maxy - bbox.miny) / mheight
 	left_pixel_center_x = bbox.minx + pixel_x_size * 0.5
-	top_pixel_center_y = bbox.maxy - pixel_y_size * 0.5
+	top_pixel_center_y = bbox.miny + pixel_y_size * 0.5
 	return ''.join(["{:.8f}\n".format(n) for n in [pixel_x_size, pixel_y_size, 0.0, 0.0, left_pixel_center_x, top_pixel_center_y]])
 
+def parse_url(url, options):
+	"""Parse map URL into options map"""
+	lat = None
+	lon = None
+	zoom = None
+	m = re.search(r'[#/=]([0-9]{1,2})/(-?[0-9]{1,2}\.[0-9]+)/(-?[0-9]{1,3}\.[0-9]+)', url)
+	if m:
+		zoom = int(m.group(1))
+		lat = float(m.group(2))
+		lon = float(m.group(3))
+	else:
+		m = re.search(r'lat=(-[0-9]{1,2}\.[0-9]+)', url, flags=re.IGNORECASE)
+		if m:
+			lat = float(m.group(1))
+		m = re.search(r'lon=(-[0-9]{1,3}\.[0-9]+)', url, flags=re.IGNORECASE)
+		if m:
+			lon = float(m.group(1))
+		m = re.search(r'zoom=([0-9]{1,2})', url, flags=re.IGNORECASE)
+		if m:
+			zoom = int(m.group(1))
+	print lat, lon, zoom
+	if zoom and not options.zoom:
+		options.zoom = zoom
+	if lat and lon and not options.center:
+		options.center = [lon, lat]
+	if not options.size and not options.size_px and not options.a and not options.fit and not options.bbox:
+		options.size_px = [1280, 1024]
+
 if __name__ == "__main__":
 	parser = argparse.ArgumentParser(description='Tile-aware mapnik image renderer')
 	parser.add_argument('-z', '--zoom', type=float, help='Target zoom level')
@@ -113,11 +140,12 @@ if __name__ == "__main__":
 	parser.add_argument('--add-layers', help='Map layers to include, comma-separated')
 	parser.add_argument('--hide-layers', help='Map layers to hide, comma-separated')
 
-	parser.add_argument('-f', '--format', dest='fmt', help='Target file format (by default looks at extension)')
+	parser.add_argument('--url', help='URL of a map to center on')
 	parser.add_argument('--ozi', type=argparse.FileType('w'), help='Generate ozi map file')
 	parser.add_argument('--wld', type=argparse.FileType('w'), help='Generate world file')
 	parser.add_argument('-t', '--tiles', type=int, choices=range(1, 13), default=1, help='Write N×N tiles, then join using imagemagick')
 	parser.add_argument('-v', '--debug', action='store_true', default=False, help='Display calculated values')
+	parser.add_argument('-f', '--format', dest='fmt', help='Target file format (by default looks at extension)')
 	parser.add_argument('style', help='Style file for mapnik')
 	parser.add_argument('output', help='Resulting image file')
 	options = parser.parse_args()
@@ -127,6 +155,9 @@ if __name__ == "__main__":
 	size = None
 	bbox = None
 
+	if options.url:
+		parse_url(options.url, options)
+
 	# format should not be empty
 	if options.fmt:
 		fmt = options.fmt.lower()
@@ -244,6 +275,12 @@ if __name__ == "__main__":
 		print 'bbox_wgs84={}'.format(transform.backward(bbox) if bbox else None)
 		print 'layers=' + ','.join([l.name for l in m.layers if l.active])
 
+	# generate metadata
+	if options.ozi:
+		options.ozi.write(prepare_ozi(bbox, size[0], size[1], options.output))
+	if options.wld:
+		options.wld.write(prepare_wld(bbox, size[0], size[1]))
+
 	# export image
 	m.aspect_fix_mode = mapnik.aspect_fix_mode.GROW_BBOX;
 	m.resize(size[0], size[1])
@@ -295,9 +332,3 @@ if __name__ == "__main__":
 			if result == 0:
 				for tile in tile_files:
 					os.remove(tile)
-
-	# generate metadata
-	if options.ozi:
-		options.ozi.write(prepare_ozi(m, options.output))
-	if options.wld:
-		options.wld.write(prepare_wld(m))
